@@ -8,6 +8,7 @@
 // Software.
 
 use log::debug;
+use std::fs;
 use std::{
     collections::HashSet,
     env,
@@ -63,8 +64,8 @@ struct CmdArgs {
     nodes_dir: PathBuf,
 
     /// Number of nodes to spawn with the first one being the genesis. This number should be greater than 0.
-    #[structopt(short = "n", long, default_value = "8")]
-    num_nodes: u8,
+    #[structopt(short = "n", long, default_value = "11", env = "NODE_COUNT")]
+    num_nodes: usize,
 
     /// Verbosity level for nodes logs (default: INFO)
     #[structopt(short = "y", long, parse(from_occurrences))]
@@ -74,6 +75,10 @@ struct CmdArgs {
     #[structopt(long = "ip")]
     ip: Option<String>,
 
+    /// IP used to launch the nodes with.
+    #[structopt(long = "add")]
+    add_nodes_to_existing_network: bool,
+
     /// Run the section locally.
     #[structopt(long = "local")]
     is_local: bool,
@@ -220,12 +225,12 @@ pub fn join_with(cmd_args: Option<&[&str]>) -> Result<(), String> {
     run_node_cmd(&node_bin_path, &current_node_args, args.verbosity, rust_log)?;
 
     let msg = format!(
-        "Node logs are being stored at: {}/sn_node_rCURRENT.log",
+        "Node logs are being stored at: {}/sn_node.log<DATETIME>",
         node_dir
     );
     if args.verbosity > 0 {
         println!("{}", msg);
-        println!("(Note that log files are rotated, and subsequent files will be named sn_node_r[NNNNN].log, with values starting at 00000 and up to 99999.");
+        println!("(Note that log files are rotated hourly, and subsequent files will be named sn_node.log<NEW DATE TINE>.");
     }
 
     Ok(())
@@ -255,6 +260,7 @@ pub fn run_with(cmd_args: Option<&[&str]>) -> Result<(), String> {
 
     let idle = args.idle_timeout_msec.to_string();
     let keep_alive = args.keep_alive_interval_msec.to_string();
+    let adding_nodes: bool = args.add_nodes_to_existing_network;
 
     common_args.push("--idle-timeout-msec");
     common_args.push(&idle);
@@ -268,32 +274,34 @@ pub fn run_with(cmd_args: Option<&[&str]>) -> Result<(), String> {
     };
 
     let rust_log = get_rust_log(args.rust_log);
-
-    // Construct genesis node's command arguments
-    let genesis_node_dir = &args.nodes_dir.join("sn-node-genesis");
-    let genesis_node_dir_str = genesis_node_dir.display().to_string();
-    let mut genesis_args = common_args.clone();
-    genesis_args.push("--first");
-    genesis_args.push(&addr);
-    let genesis_node_args =
-        build_node_args(genesis_args, &genesis_node_dir_str, None /* genesis */);
-
-    // Let's launch genesis node now
-    let msg = "Launching genesis node (#1)...";
-    if args.verbosity > 0 {
-        println!("{}", msg);
-    }
-    debug!("{}", msg);
-    run_node_cmd(
-        &node_bin_path,
-        &genesis_node_args,
-        args.verbosity,
-        rust_log.clone(),
-    )?;
-
     // Get port number of genesis node to pass it as hard-coded contact to the other nodes
     let interval_duration = Duration::from_secs(args.interval);
-    thread::sleep(interval_duration);
+
+    if !adding_nodes {
+        // Construct genesis node's command arguments
+        let genesis_node_dir = &args.nodes_dir.join("sn-node-genesis");
+        let genesis_node_dir_str = genesis_node_dir.display().to_string();
+        let mut genesis_args = common_args.clone();
+        genesis_args.push("--first");
+        genesis_args.push(&addr);
+        let genesis_node_args =
+            build_node_args(genesis_args, &genesis_node_dir_str, None /* genesis */);
+
+        // Let's launch genesis node now
+        let msg = "Launching genesis node (#1)...";
+        if args.verbosity > 0 {
+            println!("{}", msg);
+        }
+        debug!("{}", msg);
+        run_node_cmd(
+            &node_bin_path,
+            &genesis_node_args,
+            args.verbosity,
+            rust_log.clone(),
+        )?;
+
+        thread::sleep(interval_duration);
+    }
 
     // Fetch node_conn_info from $HOME/.safe/node/node_connection_info.config.
     let genesis_contact_info = read_genesis_conn_info(args.verbosity)?;
@@ -305,19 +313,44 @@ pub fn run_with(cmd_args: Option<&[&str]>) -> Result<(), String> {
         );
     }
 
+    let paths = fs::read_dir(&args.nodes_dir)
+        .map_err(|_| "Could not read existing testnet log dir".to_string())?;
+
+    let existing_nodes_count = paths
+        .collect::<Result<Vec<_>, io::Error>>()
+        .map_err(|_| "Error collecting testnet log dir".to_string())?
+        .len();
+
+    println!("{:?} existing nodes found", existing_nodes_count);
+
+    if existing_nodes_count == 0 {
+        return Err("A genesis node could not be found.".to_string());
+    }
+
+    let end: usize = if adding_nodes {
+        existing_nodes_count + args.num_nodes
+    } else {
+        args.num_nodes
+    };
+
     // We can now run the rest of the nodes
-    for i in 2..args.num_nodes + 1 {
+    for i in existing_nodes_count..end {
+        let this_node = i + 1;
         // Construct current node's command arguments
         let node_dir = &args
             .nodes_dir
-            .join(&format!("sn-node-{}", i))
+            .join(&format!("sn-node-{}", this_node))
             .display()
             .to_string();
 
         let current_node_args =
             build_node_args(common_args.clone(), &node_dir, Some(&genesis_contact_info));
 
-        let msg = format!("Launching node #{}...", i);
+        let msg = if adding_nodes {
+            format!("Adding node #{}...", this_node)
+        } else {
+            format!("Launching node #{}...", this_node)
+        };
         if args.verbosity > 0 {
             println!("{}", msg);
         }
